package com.dayou.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.dayou.dto.MajorStatisticalSelectDTO;
import com.dayou.entity.User;
import com.dayou.mapper.MajorStatisticalStatementMapper;
import com.dayou.service.IUserService;
import com.dayou.service.IMajorStatisticalStatementService;
import com.dayou.vo.MajorLedgerVO;
import com.dayou.vo.MajorPerformanceDeductionDetailVO;
import com.dayou.vo.MajorPerformanceDeductionVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class MajorStatisticalStatementServiceImpl implements IMajorStatisticalStatementService {

    @Autowired
    private IUserService userService;

    @Autowired
    private MajorStatisticalStatementMapper majorStatisticalStatementMapper;

    /**
     * 查询大中型台账
     * @param page 分页
     * @param dto 查询条件
     * @return Page<MajorLedgerVO>
     */
    @Override
    public Page<MajorLedgerVO> getMajorLedgerVO(Page page, MajorStatisticalSelectDTO dto) {
        Page<MajorLedgerVO> majorLedgerVOPage = majorStatisticalStatementMapper.getMajorLedgerVO(page, dto);
        majorLedgerVOPage.setRecords(setMembers(majorLedgerVOPage.getRecords()));
        return majorLedgerVOPage;
    }

    /**
     * 导出大中型台账
     * @param dto 查询条件
     * @return List<MajorLedgerVO>
     */
    @Override
    public List<MajorLedgerVO> exportMajorLedgerVO(MajorStatisticalSelectDTO dto) {
        List<MajorLedgerVO> majorLedgerVOList = majorStatisticalStatementMapper.exportMajorLedgerVO(dto);
        majorLedgerVOList = setMembers(majorLedgerVOList);
        return majorLedgerVOList;
    }

    /**
     * 评估部绩效扣分查询
     * @param dto 查询dto
     * @return List<MajorPerformanceDeductionVO>
     */
    @Override
    public List<MajorPerformanceDeductionVO> getMajorPerformanceDeductionVO(MajorStatisticalSelectDTO dto) {
        return majorStatisticalStatementMapper.getMajorPerformanceDeductionVO(dto);
    }

    /**
     * 评估人员绩效扣分查询
     * @param dto 查询dto
     * @return List<MajorPerformanceDeductionVO>
     */
    @Override
    public List<MajorPerformanceDeductionVO> getMajorPerformanceDeductionVOByEvaluator(MajorStatisticalSelectDTO dto) {
        return majorStatisticalStatementMapper.getMajorPerformanceDeductionVOByEvaluator(dto);
    }

    /**
     * 查询错误详情列表
     * @param page 分页
     * @param dto 查询dto
     * @return Page<MajorPerformanceDeductionDetailVO>
     */
    @Override
    public Page<MajorPerformanceDeductionDetailVO> getMajorPerformanceDeductionDetailVO(Page page, MajorStatisticalSelectDTO dto) {
        Page<MajorPerformanceDeductionDetailVO> detailVOPage = majorStatisticalStatementMapper.getMajorPerformanceDeductionDetailVO(page, dto);
        detailVOPage.setRecords(setDetailVOJSONInfo(detailVOPage.getRecords()));
        return detailVOPage;
    }

    /**
     * 导出错误详情列表
     * @param dto 查询dto
     * @return List<MajorPerformanceDeductionDetailVO>
     */
    @Override
    public List<MajorPerformanceDeductionDetailVO> exportMajorPerformanceDeductionDetailVO(MajorStatisticalSelectDTO dto) {
        List<MajorPerformanceDeductionDetailVO> detailVOList = majorStatisticalStatementMapper.exportMajorPerformanceDeductionDetailVO(dto);
        detailVOList = setDetailVOJSONInfo(detailVOList);
        return detailVOList;
    }

    /**
     * 设置项目参与人
     * @param majorLedgerVOList 台账集合
     * @return List<MajorLedgerVO>
     */
    public List<MajorLedgerVO> setMembers(List<MajorLedgerVO> majorLedgerVOList){
        // 获取用户信息
        LambdaQueryWrapper<User> lambdaQueryWrapper = new LambdaQueryWrapper<User>()
                .select(User::getId, User::getName)
                .eq(User::getDeleted, false);
        List<User> userList = userService.list(lambdaQueryWrapper);
        for (MajorLedgerVO majorLedgerVO : majorLedgerVOList){
            if (ObjectUtil.isNotNull(majorLedgerVO.getMembers())) {
                JSONArray jsonArray = JSONArray.parseArray(majorLedgerVO.getMembers());
                List<Long> membersId = jsonArray.toJavaList(Long.class);
                String names = null;
                for (User user : userList) {
                    if (membersId.contains(user.getId())) {
                        if (ObjectUtil.isNull(names)) {
                            names = user.getName() + '；';
                        } else {
                            names += user.getName() + '；';
                        }
                    }
                }
                majorLedgerVO.setMembers(names);
            }
        }
        return majorLedgerVOList;
    }

    /**
     * 设置参与人信息
     *
     * @param detailVOList 查询出来的VO信息
     * @return List<AssetsPerformanceDeductionDetailVO>
     */
    public List<MajorPerformanceDeductionDetailVO> setDetailVOJSONInfo(List<MajorPerformanceDeductionDetailVO> detailVOList) {
        LambdaQueryWrapper<User> lambdaQueryWrapper = new LambdaQueryWrapper<User>()
                .select(User::getId, User::getName)
                .eq(User::getDeleted, false);
        List<User> userList = userService.list(lambdaQueryWrapper);
        for (MajorPerformanceDeductionDetailVO detailVO : detailVOList) {
            if (ObjectUtil.isNotNull(detailVO.getMembers())) {
                JSONArray jsonArray = JSONArray.parseArray(detailVO.getMembers());
                List<Long> membersId = jsonArray.toJavaList(Long.class);
                String names = null;
                for (User user : userList) {
                    if (membersId.contains(user.getId())) {
                        if (ObjectUtil.isNull(names)) {
                            names = user.getName() + '；';
                        } else {
                            names += user.getName() + '；';
                        }
                    }
                }
                detailVO.setMembers(names);
            }
        }
        return detailVOList;
    }
}
